﻿
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Threading.Tasks;
using Wisej.Web;
using Wisej.Web.Ext.FullCalendar;

namespace TestCalendar
{
    public partial class Window1 : Form
    {
        static Random random = new Random();
        List<CalendarEvent> _events = new List<CalendarEvent>();

        public Window1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DateTime dateStart = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            DateTime dateEnd = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.DaysInMonth(dateStart.Year, dateStart.Month));
            int id = 0;

            while (dateStart <= dateEnd)
            {
                int randConfirm = random.Next(0, 10) + 1;
                bool confirmed = false;

                if (randConfirm >= 5)
                {
                    confirmed = true;
                }

                for (int h = 0; h < 23; h++)
                {
                    DateTime dateEvent = new DateTime(dateStart.Year, dateStart.Month, dateStart.Day, h, 00, 00);

                    CalendarEvent calendar = new CalendarEvent()
                    {
                        ID = id,
                        Confirmed = confirmed,
                        Date = dateEvent,
                        Color = $"#{GetRandomHexNumber(6)}"
                    };

                    _events.Add(calendar);
                    id++;

                }

                dateStart = dateStart.AddDays(1);
            }

            List<Event> events = new List<Event>();

            foreach (var item in _events)
            {
                Event even = CreateEvent(item);
                events.Add(even);
            }

            if (events != null && events.Count > 0)
            {
                fullCalendar1.Events.AddRange(events);
            }

            button2.Enabled = true;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            UpdateCalendar();
        }

        private Event CreateEvent(CalendarEvent even)
        {
            Color color = Color.Gray;

            if (even.Confirmed)
            {
                color = ColorTranslator.FromHtml(even.Color);
            }

            return new Event()
            {
                Id = even.Text,
                Start = even.Date,
                End = even.Date.AddHours(1),
                Title = even.Text,
                BackgroundColor = color
            };
        }

        public static string GetRandomHexNumber(int digits)
        {
            byte[] buffer = new byte[digits / 2];
            random.NextBytes(buffer);
            string result = String.Concat(buffer.Select(x => x.ToString("X2")).ToArray());
            if (digits % 2 == 0)
                return result;
            return result + random.Next(16).ToString("X");
        }

        private void UpdateCalendar()
        {
            //just to simulate like 3 or 4 user updating different events
            //we do it like so beacuse we need to have all the clients updated

            int randID = random.Next(_events.Count) + 1;

            lock (_events)
            {
                for (int i = 0; i < randID; i++)
                {
                    CalendarEvent calendar = _events[i];
                    calendar.Confirmed = !calendar.Confirmed;

                    _events.Remove(calendar);
                    _events.Add(calendar);

                    Application.StartTask(() =>
                    {
                        Event updatedEvent = CreateEvent(calendar);

                        if (updatedEvent != null)
                        {
                            Application.Update(this, () =>
                            {
                                try
                                {
                                    fullCalendar1.Events.Add(updatedEvent);
                                }
                                catch (Exception ex)
                                {
                                    //it never catches the error 
                                    Console.WriteLine(ex);
                                }
                            });
                        }
                    });
                }
            }

        }
    }

    public class CalendarEvent
    {
        public int ID { get; set; }

        public DateTime Date { get; set; }

        public bool Confirmed { get; set; }

        public string Text { get { return ID.ToString(); } }

        public string Color { get; set; }
    }
}
